from sys import stdin
import sys
from pysh.lib.exec import Exec


def generate(n):
    for i in range(int(n)):
        print(f"{i} {i+1}")

def consume():
    for line in stdin:
        a, b = line.split(' ')
        print(int(a) + int(b))

def add(a, b):
    print(int(a) + int(b))

def hello():
    print("hello world")

def fail():
    print("fail")
    exit(-1)


if __name__ == '__main__':
    test_chain = ["generate 10", "consume", "fail", "hello", "add $[2,0] $[2,1]"]

    test_cmd = ["python test.py " + x for x in test_chain]
    if len(sys.argv) == 1:
        t = Exec("python test.py generate 10")
        print(t.stdout())
        print("abcdef")
        # python .\test.py g 10 | python .\test.py c
        t = Exec(test_cmd[0]) | Exec(test_cmd[1])
        print(t._cmd)
        print(t.stdout())

        t = Exec(test_cmd[0]) | Exec(test_cmd[4])
        # print(t._cmd)
        print(t.stdout())

        t1 = Exec(test_cmd[2])
        t2 = Exec(test_cmd[3])
        t2.run_if_fail(t1)
        print(t2.stdout())
    else:
        this_module = sys.modules[__name__]
        getattr(this_module, sys.argv[1])(*sys.argv[2:])




import sys
import time

from pysh.lib.filter import Filter, RegexFilter, FuncFilter
from pysh.lib.exec import Exec

def generate():
    for i in range(5):
        print(f"{i} {i + 1} {time.time()}")
        time.sleep(1)

def consume():
    for line in sys.stdin:
        a, b, c = line.split(' ')
        print(f"{int(a) + int(b)} {float(c)} {time.time()}")


if __name__ == '__main__':
    test_chain = ["generate", "consume"]
    test_cmd = ["python .\\test_stream.py " + x for x in test_chain]

    if len(sys.argv) == 1:
        g = Exec(test_cmd[0])
        def _f(x: str):
            print("FFF", x, time.time())
            return x
        f = FuncFilter(_f)
        # f.set_stream(g)
        g.stream_to(f)
        # print(g._state, f._state)
        print(f.stdout())
        # python .\test.py g 10 | python .\test.py c
        g = Exec(test_cmd[0])
        f = Exec(test_cmd[1])
        # f.set_stream(g)
        g.stream_to(f)
        print(g._state, f._state)
        print(f.stdout().decode())

        g = Exec(test_cmd[0])
        f = Exec(test_cmd[1])
        f.set_input(g)
        print(g._state, f._state)
        print(f.stdout().decode())
    else:
        this_module = sys.modules[__name__]
        getattr(this_module, sys.argv[1])(*sys.argv[2:])



from lib.pack_name import PackName
from pysh.lib.exec import Exec
from lib.generate_param import Param, generate_param

d = {
    'run': Param.COMMAND,
    'it': Param.SHORTCUT,
    'rm': None,
    'privileged': None,
    'mount': (
        {
            'type': 'bind',
            'source': '/home/abc/Document',
            'target': '/Doc'
        }, {
            'type': 'bind',
            'source': '/home/abc/Document1',
            'target': '/Doc1'
        },
    ),
    'nginx:v1.0': Param.COMMAND
}

# print(Exec(f"C:\\tools\\echo \"{generate_param(d)}\"").stdout())

_pysht_19_=Exec(f"python test.py generate 10")
a=_pysht_19_
print("run a")
_pysht_21_=PackName(a)
_pysht_20_=Exec(f"python test.py consume")
_pysht_20_.set_input(_pysht_21_)
b=_pysht_20_
_pysht_23_=PackName(a)
_pysht_22_=Exec(f"python test.py consume")
_pysht_22_.set_input(_pysht_23_)
c=_pysht_22_
print(b.stdout())
print(c.stdout())



import re
import subprocess
from typing import Callable, Optional, Union

import enum

from pysh.lib.FakeFile import FilterFile


class Filter:
    class State(enum.Enum):
        NOT_START = 0
        RUNNING = 1
        FINISHED = 2
        NOT_RUN = 3
    FUNC = 1
    IF_SUCCESS = 2
    IF_FAIL = 3
    PIPE = 4
    FILTER = 5
    STREAM = 6

    def __init__(self, type, upstream: Optional['Filter']):
        self._upstream = upstream
        self._type = type
        self._stdout = None
        self._stderr = None
        self._stream = False

    def stdout(self):
        if self._stdout:
            return self._stdout
        self._stdout = self._upstream.stdout()
        return self._stdout

    def stderr(self):
        if self._stderr:
            return self._stderr
        self._stderr = self._upstream.stderr()
        return self._stderr

    def returncode(self):
        return self._upstream.returncode()

    def set_input(self, ano):
        self._upstream = ano

    def result(self) -> subprocess.CompletedProcess:
        return self._upstream.result()

    @property
    def state(self) -> int:
        return self._upstream.state

    @property
    def process(self) -> subprocess.Popen:
        return self._upstream.process

    @property
    def type(self):
        return self._type

    def outfile(self):
        raise RuntimeError("can not get out file")

    def errfile(self):
        raise RuntimeError("can not get err file")

    def set_stream(self, ano: 'Filter'):
        self._stream = True
        self.set_input(ano)

    def join(self):
        self._upstream.join()

    def __str__(self):
        return str(self.result())

    def __repr__(self):
        return str(self)


class FuncFilter(Filter):
    def __init__(self, func: Callable[[Optional[Union[subprocess.CompletedProcess, str]]], str] = None,
                 upstream: Optional['Filter'] = None):
        super().__init__(Filter.FUNC, upstream)
        self._func = func
        self._stdout = None

    def stdout(self):
        if not self._stdout:
            if not self._stream:
                self._stdout = self(self._upstream.result())
            else:
                res = []
                file = self.outfile()
                while True:
                    line = file.readline()
                    if line == b'' or line == '':
                        break
                    res.append(line)
                self._stdout = b''.join(res)

        return self._stdout

    def stderr(self):
        if not self._stderr:
            if not self._stream:
                self._stderr = self(self._upstream.result().stderr)
            else:
                res = []
                file = self.errfile()
                while True:
                    line = file.readline()
                    if line == b'' or line == '':
                        break
                    res.append(line)
                if len(res) > 0 and isinstance(res[0], bytes):
                    self._stderr = b''.join(res)
                else:
                    self._stderr = ''.join(res)

        return self._stderr

    def __call__(self, arg):
        return self._func(arg)

    def outfile(self):
        return FilterFile(self._upstream.outfile(), self)

    def errfile(self):
        return FilterFile(self._upstream.errfile(), self)


class RegexFilter(FuncFilter):
    def __init__(self, reg):
        super().__init__(None)

        def _filter(result: Optional[Union[subprocess.CompletedProcess, str]]):
            if result is None:
                return reg
            if isinstance(result, subprocess.CompletedProcess):
                result = result.stdout.decode()
            pat = re.compile(reg)
            res = pat.findall(result)
            return '\n'.join(map(lambda x: x if isinstance(x, str) else ' '.join(x), res))

        self._func = _filter



from sys import stdin
import sys

from pysh.lib.filter import Filter, RegexFilter
from pysh.lib.exec import Exec

a = """
total 17
-rw-r--r-- 1 wsl wsl 1093 Aug 31 11:17 1.ps
-rw-r--r-- 1 wsl wsl  552 Aug 31 11:17 2.ps
-rw-r--r-- 1 wsl wsl  437 Aug 31 11:17 3.ps
-rw-r--r-- 1 wsl wsl  199 Aug 27 11:58 a.c
-rw-r--r-- 1 wsl wsl 1880 Aug 27 11:58 a.o
-rw-r--r-- 1 wsl wsl 5360 Aug 31 11:39 temp.ps
"""

def generate():
    print(a)


if __name__ == '__main__':
    test_chain = ["generate"]
    test_cmd = ["python .\\test_filter.py " + x for x in test_chain]

    if len(sys.argv) == 1:
        # python .\test.py g 10 | python .\test.py c
        g = Exec(test_cmd[0])
        f = RegexFilter(r".+\.ps")
        f.set_input(g)
        print(f.stdout())
    else:
        this_module = sys.modules[__name__]
        getattr(this_module, sys.argv[1])(*sys.argv[2:])



from .excepts import NoneOfMyBusiness
from .parser2 import start as st2

def start(tokens):
    ops = [st2]
    sss = 0
    while True:
        for i in range(len(ops)):
            op = ops[i]
            try:
                res = op(tokens)
                sss = 0
                for i in res:
                    yield i
            except NoneOfMyBusiness:
                sss += 1
                if sss == len(ops):
                    for i in range(len(tokens)):
                        yield tokens[i]
                    tokens.clear()
                    sss = 0
                else:
                    continue
                # if i < len(ops) - 1:
                #     continue
                # for i in range(len(tokens)):
                #     yield tokens[i]
                # tokens.clear()
            except StopIteration:
                return None




import subprocess
import enum
import threading
from typing import List, Callable, Union, Optional

from pysh.lib.FakeFile import RealFile, FakeFile
from pysh.config import Config
import logging as log
import re

from pysh.lib.filter import Filter, RegexFilter, FuncFilter


class ResultTable:
    def __init__(self, input):
        self._input = input
        self._lines = None
        self._arr = None

    def __getitem__(self, item):
        if isinstance(item, int):
            i = item
            j = None
        else:
            i, j = item
        if j is None:
            if self._lines is None:
                self._lines = self._input.stdout().decode().split('\n')
                self._arr = [None for _ in self._lines]
            return self._lines[i]
        if self._arr is None or self._arr[i] is None:
            line = self[i].split()
            self._arr[i] = line
        return self._arr[i][j]

    def set_input(self, input):
        self._input = input


class Exec(Filter):

    def __init__(self, cmd: str, upstream=None, capture_output=False, pre=None):
        super().__init__(Filter.PIPE, upstream)
        if pre is None:
            pre = []
        self._cmd = cmd
        self._state = Exec.State.NOT_START
        self._result = None
        self._process = None
        self._upstream = upstream
        self._capture_output = capture_output
        self._pre = pre

    def __exec_func(self):
        self._state = Exec.State.RUNNING
        input_file: FakeFile = self._upstream.outfile() if self._stream else None
        if not input_file:
            if self._upstream is None:
                input_str = None
            elif isinstance(self._upstream, Filter):
                if self._upstream.type in (Filter.PIPE, Filter.FUNC, Filter.FILTER):
                    if self._upstream.returncode() != 0:
                        log.debug(f"pipe upstream ({self._upstream}) fail")
                        self._state = self.State.NOT_RUN
                        self._result = self._upstream.result()
                        return None
                    input_str = self._upstream.stdout()
                elif self._upstream.type == Filter.IF_SUCCESS:
                    if self._upstream._upstream.returncode() != 0:
                        log.debug(f"logic and upstream ({self._upstream}) fail")
                        self._state = self.State.NOT_RUN
                        # self._result = subprocess.CompletedProcess(self._cmd, self._input.upstream.returncode())
                        self._result = self._upstream.result()
                        return None
                    input_str = None
                elif self._upstream.type == Filter.IF_FAIL:
                    if self._upstream._upstream.returncode() == 0:
                        log.debug(f"logic or upstream ({self._upstream}) success")
                        self._state = self.State.NOT_RUN
                        # self._result = subprocess.CompletedProcess(self._cmd, self._input.upstream.returncode())
                        self._result = self._upstream.result()
                        return None
                    input_str = None
                else:
                    input_str = None
            elif isinstance(self._upstream, (str, bytes)):
                input_str = self._upstream
            else:
                raise Exception(f"type {type(self._upstream)} is not supported as input")
            log.debug("start run " + self._cmd)
            log.debug("with input " + str(input_str))
            self._cmd = self._parse_exec_cmd(self._cmd, self._upstream)
            self._process = subprocess.Popen(self._cmd, shell=True,
                                             stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            if isinstance(input_str, str):
                input_str = input_str.encode()
            if input_str:
                self._process.stdin.write(input_str)
        else:
            # 流
            log.debug("start run stream " + self._cmd)
            self._process = subprocess.Popen(self._cmd, shell=True,
                                             stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

            def _pass():
                while True:
                    line = input_file.readline()
                    if line in ('', b''):
                        self._state = Exec.State.FINISHED
                        break
                    if isinstance(line, str):
                        line = line.encode()
                    self._process.stdin.write(line)
                    self._process.stdin.flush()
                    log.debug(f"write {line} to process")
                log.debug("quit thread")
            threading.Thread(target=_pass).start()

    def exec(self):
        if self._state != Exec.State.NOT_START:
            return

        if Config.run_in_thread:
            threading.Thread(target=Exec.__exec_func, args=(self,)).start()
        else:
            self.__exec_func()

    def join(self):
        # while self._state not in (Exec.State.FINISHED, Exec.State.NOT_RUN):
        #     pass
        if self._result:
            return
        if self._upstream:
            self._upstream.join()
        # stdout, stderr = self._process.communicate()
        self._process.stdin.close()
        retc = None
        while retc is None:
            retc = self._process.poll()
        log.debug(f"process {self._process.pid} finish with " + str(retc))
        stdout = self._process.stdout.read()
        stderr = self._process.stderr.read()
        # log.debug("process finish with " + str(retc))
        self._result = subprocess.CompletedProcess(self._cmd, self._process.returncode, stdout, stderr)
        self._state = self.State.FINISHED

    def stdout(self):
        self.exec()
        self.join()
        return self._result.stdout

    def stderr(self):
        self.exec()
        self.join()
        return self._result.stderr

    def returncode(self):
        self.exec()
        self.join()
        return self._result.returncode

    def result(self):
        self.exec()
        self.join()
        return self._result

    @property
    def state(self):
        return self._state

    @property
    def process(self):
        return self._process

    @property
    def input(self):
        return self._upstream

    def outfile(self):
        if self._state == Exec.State.NOT_START:
            self.exec()
        return RealFile(self._process.stdout)

    def errfile(self):
        if self._state == Exec.State.NOT_START:
            self.exec()
        return RealFile(self._process.stderr)

    def __or__(self, other):
        other.set_input(self)
        return other

    def __ror__(self, other):
        if isinstance(other, (str, Exec)):
            self._upstream = other
        return self

    def run_if_fail(self, ano: 'Exec'):
        self._upstream = Filter(Filter.IF_FAIL, ano)

    def run_if_success(self, ano: 'Exec'):
        self._upstream = Filter(Filter.IF_SUCCESS, ano)

    def set_input(self, other):
        self._upstream = other
        return other

    def set_stream_input(self, other):
        self.set_stream(other)

    def _parse_exec_cmd(self, cmd, input: Union[Filter, 'Exec']):
        arr = ResultTable(input)
        ans = ''
        state = 0
        i = None
        j = None
        for x in cmd:
            if state == 0:
                if x == '$':
                    state = 1
                else:
                    ans += x
            elif state == 1:
                if x == '$':
                    ans += '$'
                    state = 0
                elif x == '[':
                    state = 2
                    i = 0
                else:
                    ans += f'${x}'
                    state = 0
            elif state == 2:
                if str.isdigit(x):
                    i = i * 10 + int(x)
                elif x == ',':
                    j = 0
                    state = 3
                elif x == ']':
                    ans += arr[i]
                    state = 0
                elif x == ' \t':
                    pass
                else:
                    raise RuntimeError("2222")
            elif state == 3:
                if str.isdigit(x):
                    j = j * 10 + int(x)
                elif x == ']':
                    ans += arr[i, j]
                    state = 0
                elif x in ' \t':
                    pass
                else:
                    raise RuntimeError("3333")
        return ans

    def pipe_to(self, ano: Union["Filter", "Exec", str, Callable[[Optional[subprocess.CompletedProcess]], str]]):
        if isinstance(ano, str):
            ano = RegexFilter(ano)
        elif callable(ano):
            ano = FuncFilter(ano)
        ano.set_input(self)
        return ano

    def stream_to(self, ano):
        ano.set_stream(self)
        return ano

    def success_to(self, ano: "Exec"):
        ano.run_if_success(self)
        return ano

    def fail_to(self, ano: "Exec"):
        ano.run_if_fail(self)
        return ano



from enum import Enum


class Param(Enum):
    SHORTCUT = 1
    COMMAND = 2
    ARGUMENT = 3


def generate_list(p, prefix, delimiter):
    return delimiter.join([prefix + x for x in p])


def generate_dict(d: dict, prefix, delimiter, shortcut_prefix, assignment):
    ans = []
    for k, v in d.items():
        p = None
        if isinstance(v, Param):
            if v == Param.SHORTCUT:
                p = shortcut_prefix
            elif v == Param.COMMAND:
                p = ''
        if p is None:
            p = shortcut_prefix if len(k) == 1 else prefix

        def generate_value(value):
            if value is None or isinstance(value, Param):
                return f"{p}{k}"
            elif isinstance(value, str):
                return f"{p}{k}{assignment}{value}"
            elif isinstance(value, list):
                return f"{p}{k}{assignment}{generate_list(value, '', ',')}"
            elif isinstance(value, dict):
                return f"{p}{k}{assignment}{generate_dict(value, '', ',', '', '=')}"

        if isinstance(v, tuple):
            ans += [generate_value(x) for x in v]
        else:
            ans.append(generate_value(v))
    return delimiter.join(ans)


def generate_param(p, prefix=None, delimiter=None, shortcut=None):
    if isinstance(p, str):
        prefix = '-' if not prefix else prefix
        delimiter = ' ' if not delimiter else delimiter
        return generate_list(p, prefix, delimiter)
    if isinstance(p, dict):
        prefix = '--' if not prefix else prefix
        delimiter = ' ' if not delimiter else delimiter
        shortcut = '-' if not shortcut else shortcut
        return generate_dict(p, prefix, delimiter, shortcut, ' ')
    return str(p)

def generateParam(*args, **kwargs):
    return generate_param(*args, **kwargs)


if __name__ == '__main__':
    d = {
        'run': Param.COMMAND,
        'it': Param.SHORTCUT,
        'rm': None,
        'privileged': None,
        'mount': (
            {
                'type': 'bind',
                'source': '/home/abc/Document',
                'target': '/Doc'
            }, {
                'type': 'bind',
                'source': '/home/abc/Document1',
                'target': '/Doc1'
            },
        ),
        'nginx:v1.0': Param.COMMAND
    }

    print(generate_param(d, prefix='--', shortcut='-'))



import tokenize
from typing import Tuple

from .token import Line, TokenList, Token, TokenGenerator
from .excepts import NoneOfMyBusiness


def start(token: TokenGenerator):
    line = token.get_next_line()
    t = ParseStatement(line).check_e_statement_line()
    token.clear()
    return t


class ParseStatement:
    def __init__(self, line: Line, start: int = 0):
        self._line = line
        self._cur = start

    def check_e_able(self) -> Tuple[bool, int]:
        assignment = -1
        has_e_obj = False
        for i in range(len(self._line)):
            x = self._line[i]
            if x.type == tokenize.OP and x.value == '=':
                if assignment == -1:
                    assignment = True
            if x.type == tokenize.NAME and x.value in ('e', 'g') and self._line[i + 1].type == tokenize.STRING:
                has_e_obj = True
                break
            if x.type == tokenize.ERRORTOKEN and x.value == '`':
                has_e_obj = True
                break
        return has_e_obj, assignment

    def check_e_statement_line(self) -> TokenList:
        has_e_object, assignment = self.check_e_able()
        if not has_e_object:
            raise NoneOfMyBusiness()

        result = []
        i = 0
        back_brackets = 0
        while i < len(self._line):
            x = self._line[i]
            if assignment > 0:
                result.append(x)
                if x.value == '=':
                    assignment = -1
                i += 1
                continue

            tk = None
            if x.type == tokenize.NAME and x.value == 'e' and self._line[i + 1].type == tokenize.STRING:
                self._line[i + 1].value = self.generate_cmd_str(self._line[i + 1].value)
                tk = 'Exec'
            elif x.type == tokenize.NAME and x.value == 'g' and self._line[i + 1].type == tokenize.STRING:
                tk = 'Filter'
            elif x.type == tokenize.NAME:
                tk = 'PackName'
            elif x.type == tokenize.OP and x.value == '|' and \
                    self._line[i + 1].type == tokenize.OP and self._line[i + 1].value == '|':
                tk = '.fail_to'
                i += 1
            elif x.type == tokenize.OP and x.value == '&' and \
                    self._line[i + 1].type == tokenize.OP and self._line[i + 1].value == '&':
                tk = '.success_to'
                i += 1
            elif x.type == tokenize.OP and x.value == '|':
                tk = '.pipe_to'
            elif x.type == tokenize.OP and x.value == '~':
                tk = '.stream_to'

            if tk:
                result += [
                    Token(tokenize.NAME, tk, start=self._line[i].start),
                    Token(tokenize.OP, '(', end=self._line[i + 1].start),
                ]
                if tk == 'PackName':
                    result.append(x)
                    result += [Token(tokenize.OP, ')', end=x.end)] * (back_brackets + 1)
                    back_brackets = 0
                else:
                    back_brackets += 1
            else:
                result.append(x)
                result += [Token(tokenize.OP, ')', end=x.end)] * back_brackets
                back_brackets = 0

            i += 1
        return TokenList(result, indent=self._line.indent)


    def generate_cmd_str(self, cmd: str):
        res = "f"
        state = 1
        for x in cmd:
            if state == 1:
                if x == '$':
                    state = 2
                else:
                    res += x
            elif state == 2:
                if x == '$':
                    state = 1
                    res += x
                elif x == '{':
                    state = 4
                elif str.isdigit(x):
                    state = 1
                    res += f"${x}"
                elif str.isalpha(x) or x == '_':
                    state = 3
                    res += f'{{generateParam({x}'
                else:
                    state = 1
                    res += x
            elif state == 3:
                if str.isalnum(x) or x == '_':
                    res += x
                else:
                    res += f')}}{x}'
                    state = 1
            elif state == 4:
                assert x != '}'
                res += f'{{generateParam({x}'
                state = 5
            elif state == 5:
                if x == '}':
                    state = 1
                    res += ')}'
                else:
                    res += x
        return res



import logging


logging.basicConfig(
    level=logging.INFO,
    format='%(thread)d-%(levelname)s %(asctime)s %(filename)s[%(funcName)s](%(lineno)d): %(message)s',
    datefmt='%H:%M:%S',
)


class Config:
    run_in_thread = False



import io


class FakeFile:
    def __init__(self):
        pass

    def readline(self):
        pass


class RealFile(FakeFile):
    def __init__(self, fd: io.TextIOWrapper):
        super().__init__()
        self._fd = fd

    def readline(self):
        return self._fd.readline()


class FilterFile(FakeFile):
    def __init__(self, upstream: FakeFile, node):
        super(FilterFile, self).__init__()
        self._fd = upstream
        self._node = node

    def readline(self):
        return self._node(self._fd.readline())



import tokenize
from io import StringIO
import sys
from . import parser

TEXT_TYPE = 'unicode' if sys.version_info[0] == 2 else 'str'


class InterpolationError(tokenize.TokenError): pass


def TK(token, value):
    return tokenize.TokenInfo(token, value, (0, 0), (0, 0), '')


out = open('err.txt', 'w')
def pysh_untokenize(tokens):
    parts = []
    prev_row = 1
    prev_col = 0
    last = None
    indent = 0
    for token in tokens:
        ttype, tvalue, tstart, tend, tline = token.to_token()
        print(token, file=out)
        row, col = tstart

        # if last and last.type in (tokenize.NEWLINE, tokenize.NL):
        #     parts.append('\t' * indent)
        # if ttype == tokenize.INDENT:
        #     indent += 1
        # elif ttype == tokenize.DEDENT:
        #     indent -= 1

        # Add whitespace
        col_offset = col - prev_col
        if col_offset > 0:
            parts.append(" " * col_offset)
            # parts.append(' ')
        # elif last and (last.type, token.type) in (tokenize.NAME, tokenize.NUMBER):
        #     parts.append(' ')

        # if ttype not in (tokenize.INDENT, tokenize.DEDENT):
        parts.append(tvalue)
        prev_row, prev_col = tend

        if ttype in (tokenize.NL, tokenize.NEWLINE):
            prev_row += 1
            prev_col = 0
        if ttype not in (tokenize.INDENT, tokenize.DEDENT):
            last = token
    head = """
from pysh.lib.exec import Exec
from pysh.lib.filter import Filter
from pysh.lib.pack_name import PackName
from pysh.lib.generate_param import generateParam
from pysh.lib.template import check_switch_case_condition

"""
    return head + ''.join(parts)


def get_temp():
    i = 1
    while True:
        yield f"_pysh_temp{i}__"
        i += 1


def indents(n):
    for i in range(n):
        yield tokenize.TokenInfo(tokenize.INDENT, '\t', (0, 0), (0, 0), '')


def pysh_tokenize(readline):
    tokens = tokenize.generate_tokens(readline)
    parse = parser.statement(tokens)
    # print(list(parse))
    return parse
    indent = 0
    last_obj = None
    gen_name = get_temp()
    try:
        while True:
            token = next(tokens)
            print(token)
            ttype, tvalue, tstart, tend, tline = token
            if ttype == tokenize.NAME and tvalue == 'e':
                tnext = next(tokens)
                print(tnext)
                if tnext[0] == tokenize.STRING and tnext[2][1] == tend[1]:
                    # 解析e字符串
                    last_obj = next(gen_name)
                    # yield tokenize.TokenInfo(tokenize.NAME, last_obj, token[2], (0, 0), '')
                    # yield tokenize.TokenInfo(tokenize.OP, '=', (0, 0), (0, 0), '')
                    yield tokenize.TokenInfo(tokenize.NAME, 'Exec', token[2], (0, 0), '')
                    yield tokenize.TokenInfo(tokenize.OP, '(', (0, 0), (0, 0), '')
                    yield tokenize.TokenInfo(tokenize.STRING, tnext[1], (0, 0), (0, 0), '')
                    yield tokenize.TokenInfo(tokenize.OP, ')', (0, 0), tnext[3], '')
                    # yield tokenize.TokenInfo(tokenize.NEWLINE, '\n', (0, 0), tnext[3], '')
                    # for x in indents(indent):
                    #     yield x
                    continue
                else:
                    # e后面不是紧跟字符串，原样返回
                    yield token
                    yield tnext
                    continue
            elif ttype == tokenize.INDENT:
                indent += 1
            elif ttype == tokenize.NEWLINE:
                indent = 0
            yield token
    except (StopIteration, tokenize.TokenError):
        pass



import tokenize
from typing import List, Generator, TypeVar, NewType
import logging as log


out = open('out.txt', 'w')

def _search_indents(line):
    indents = 0
    spaces = 0
    for c in line:
        if c == '\t':
            indents += 1
        elif c == ' ':
            if spaces == 3:
                indents += 1
                spaces = 0
            else:
                spaces += 1
        else:
            assert False
    return indents


class Token:

    def __init__(self, type, value:str, start=(0, 0), end=(0, 0), line=None):
        assert not isinstance(value, Token)
        assert value is not None
        self.type = type
        self.value = value
        self.start = start
        self.end = end
        self.line = line

    @staticmethod
    def from_tokenize(token: tokenize.TokenInfo):
        self = Token(None, '')
        self.type = token[0]
        self.value = token[1]
        self.start = token[2]
        self.end = token[3]
        self.line = token[4]
        return self

    def to_token(self):
        return tokenize.TokenInfo(self.type, self.value, self.start, self.end, self.line)

    def __sub__(self, other):
        assert isinstance(other, Token)
        return other.end[0] - self.start[0], other.end[1] - self.start[1]

    def __str__(self):
        return str(self.to_token())

    def __repr__(self):
        return str(self)


class Line:
    def __init__(self):
        self._data: List[Token] = []
        self._indent = 0

    def add_token(self, token: Token):
        if token.type not in (tokenize.INDENT, tokenize.DEDENT):
            self._data.append(token)

    @property
    def indent(self):
        return self._indent

    # @indent.setter
    # def indent(self, val):
    #     self._indent = val

    @property
    def data(self):
        return self._data

    def __len__(self):
        return len(self._data)

    def __iter__(self):
        return iter(self._data)

    def __getitem__(self, item):
        return self._data[item]


class TokenGenerator:
    def __init__(self, tokens: Generator[tokenize.TokenInfo, None, None]):
        self._tokens = tokens
        # 已经取得但还未生成目标代码的token
        self._pre = []
        self._indent = 0

    def __getitem__(self, item: int) -> Token:
        while item >= len(self._pre):
            new = Token.from_tokenize(next(self._tokens))
            print(new, file=out)
            # if new.type == tokenize.NEWLINE:
            #     self._indent = 0
            # elif new.type == tokenize.INDENT:
            #     self._indent += _search_indents(new.value)
            if new.type == tokenize.INDENT:
                self._indent += 1
            elif new.type == tokenize.DEDENT:
                self._indent -= 1
            self._pre.append(new)
        return self._pre[item]

    def clear(self, i=None):
        if i:
            self._pre = self._pre[i:]
        else:
            self._pre.clear()

    def push_front(self, item):
        if isinstance(item, TokenList):
            self._pre = item._data + self._pre
        else:
            self._pre.insert(0, item)

    def keep_last_one(self):
        self.clear(len(self) - 1)

    def __len__(self):
        return len(self._pre)

    @property
    def indent(self):
        return self._indent

    def get_next_line(self):
        i = 0
        line = Line()
        while True:
            line.add_token(self[i])
            if self[i].type in (tokenize.NEWLINE, tokenize.ENDMARKER, tokenize.NL):
                break
            i += 1
        line._indent = self.indent
        return line


def _temp_generator():
    i = 0
    while True:
        i += 1
        yield f"_pysht_{i}_"


temp_name = _temp_generator()


class TokenList:
    Exec = 1
    Filter = 2
    Normal = 3
    FULL = 4
    SWITCH = 5

    def __init__(self, data, indent, type=3):
        self._data = list(data)
        self._var_name = None
        self._indent = indent
        self._last_var_name = None
        self._generated_name = False
        self.type = type

        last = None
        for x in self._data:
            if not last or last.type in (tokenize.NL, tokenize.NEWLINE):
                if x.start[1] != indent * 4:
                    x.start = (x.start[1], indent * 4)
            last = x
        # if self._data:
        #     self._data = [Token(tokenize.INDENT, '    ' * indent, start=(data[0].start[0], 0), end=(data[0].end[0], 4 * indent))] + self._data

    def push_front(self, data):
        assert isinstance(data, (list, TokenList))
        start = self._data[0].start
        self._data[0].start = (0, 0)
        if isinstance(data, list):
            self._data = data + self._data
        elif isinstance(data, TokenList):
            # self._data = data._data + self._data
            data.push_back(self)
            # self = data  # 魔鬼操作
            return data
        self._data[0].start = start
        return self

    def push_back(self, data):
        assert isinstance(data, (list, TokenList))
        if self._data:
            end = self._data[-1].end
            self._data[-1].end = (0, 0)
        else:
            end = (0, 0)
        if isinstance(data, list):
            self._data = self._data + data
        else:
            self._data = self._data + data._data
            # first = True
            # for i in range(len(data._data)):
            #     x = data._data[i]
            #     if first and data.indent > self._indent:
            #         self._data += [Token(tokenize.INDENT, '\t')] * (data.indent - self._indent)
            #         first = False
            #     if x.type == tokenize.NEWLINE:
            #         self._data.append(x)
            #         if i != len(data._data) - 1:
            #             self._data += [Token(tokenize.INDENT, '\t')] * max(data.indent - self._indent, 0)
            #     else:
            #         self._data.append(x)
        if self._data:
            self._data[-1].end = end
        return self

    def push_line(self, newline: 'TokenList'):
        # self.push_back([Token(tokenize.INDENT, '\t')] * self.indent)
        self.push_back(newline)
        return self

    def eol(self):
        self._data.append(Token(tokenize.NEWLINE, '\n'))
        return self

    def newline(self):
        if not self._data:
            return
        x_offset = self._data[0].start[1]
        for x in self._data:
            x.start = (x.start[0], x.start[1] - x_offset)
            x.end = (x.end[0], x.end[1] - x_offset)
        return self

    def __iter__(self):
        return iter(self._data)

    def __getitem__(self, item: int):
        return self._data[item]

    @property
    def var_name(self):
        if not self._var_name:
            self.generate_name()
        if not self._last_var_name:
            self._last_var_name = self._var_name
        return self._var_name

    def generate_name(self):
        self._var_name = next(temp_name)
        self._generated_name = True

    @var_name.setter
    def var_name(self, value):
        self._var_name = value
        if self._last_var_name is None:
            self._last_var_name = value

    @property
    def have_name(self):
        return self._var_name is not None

    @property
    def indent(self):
        return self._indent

    @indent.setter
    def indent(self, value):
        self._indent = value

    @property
    def last_var_name(self):
        return self._last_var_name

    @last_var_name.setter
    def last_var_name(self, value):
        # print("set last var name " + value)
        self._last_var_name = value

    @property
    def generated_name(self):
        return self._generated_name




from setuptools import setup, find_packages
from distutils.sysconfig import get_python_lib

'''
    运行python setup.py check查看setup.py中的问题，添加了url、author、author_email
    执行python setup.py build
    执行python setup.py install
'''
SITE_PACKAGES_PATH = get_python_lib()
print(SITE_PACKAGES_PATH)
setup(
    name="pysh-run",
    version='1.1',
    #download_url='https://github.com/w-mj/Pysh',
    url='https://github.com/w-mj/Pysh',
    author='w-mj && lolydleo',
    author_email='wmj@alphamj.cn',
    packages=["pysh", "pysh.codec", "pysh.lib"],
    license='MIT',
    description="Run program like bash scripts.",
    long_description=open('README.md', encoding='utf-8').read(),
    keywords='run program bash',
    data_files=[(SITE_PACKAGES_PATH, ['pysh.pth'])],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)


from pysh.lib.exec import Exec
from pysh.lib.filter import RegexFilter, FuncFilter

def PackName(obj):
    if isinstance(obj, Exec):
        return obj
    if isinstance(obj, str):
        return RegexFilter(obj)
    if callable(obj):
        return FuncFilter(obj)
    return obj



#!/usr/bin/env python


import codecs, io, encodings
import sys
import traceback
import tokenize

from . import parser
from . import token
from encodings import utf_8
from .tokenizer import pysh_tokenize, pysh_untokenize


def pysh_transform(stream):
    ##output = pysh_untokenize(pysh_tokenize(stream.readline))
    ##return output.rstrip()
    try:
        tokens = tokenize.generate_tokens(stream.readline)
        tokens = token.TokenGenerator(tokens)
        parse = parser.start(tokens)
        output = pysh_untokenize(parse)
    except Exception as ex:
        raise ex
    return output

def pysh_transform_string(input):
    stream = io.StringIO(input)
    return pysh_transform(stream)

def pysh_decode(input, errors='strict'):
    return pysh_transform_string(input), len(input)

class PyshIncrementalDecoder(utf_8.IncrementalDecoder):
    def decode(self, input, final=False):
        self.buffer += input
        if final:
            buff = self.buffer
            self.buffer = b''
            return super(PyshIncrementalDecoder, self).decode(
                pysh_transform_string(buff), final=True)
        else:
            return ''

class PyshStreamReader(utf_8.StreamReader):
    def __init__(self, *args, **kwargs):
        codecs.StreamReader.__init__(self, *args, **kwargs)
        self.stream = io.StringIO(pysh_transform(self.stream))

def search_function(encoding):
    # print(encoding)
    # if encoding != 'pysh' : return None
    # Assume utf8 encoding
    utf8 = encodings.search_function('utf8')
    return codecs.CodecInfo(
        name='pysh',
        encode=utf8.encode,
        decode=pysh_decode,
        incrementalencoder=utf8.incrementalencoder,
        incrementaldecoder=PyshIncrementalDecoder,
        streamreader=PyshStreamReader,
        streamwriter=utf8.streamwriter)

codecs.register(search_function)

_USAGE = """\
Wraps a python command to allow it to recognize pysh-coded files with
no source modifications.

Usage:
    python -m pysh.codec.register -m module.to.run [args...]
    python -m pysh.codec.register path/to/script.py [args...]
"""


if __name__ == '__main__':
    if len(sys.argv) >= 3 and sys.argv[1] == '-m':
        mode = 'module'
        module = sys.argv[2]
        del sys.argv[1:3]
    elif len(sys.argv) >= 2:
        mode = 'script'
        script = sys.argv[1]
        sys.argv = sys.argv[1:]
    else:
        print(_USAGE, file=sys.stderr)
        sys.exit(1)

    if mode == 'module':
        import runpy
        runpy.run_module(module, run_name='__main__', alter_sys=True)
    elif mode == 'script':
        with open(script) as f:
            global __file__
            __file__ = script
            # Use globals as our "locals" dictionary so that something
            # that tries to import __main__ (e.g. the unittest module)
            # will see the right things.
            exec(f.read(), globals(), globals())


#
# class Template:
#     def __init__(self, reg):
#         self._reg = reg
#         self.var_t = 2
#
#     def add(self):
#         setattr(self, 'ttt', '123')
#
#
# class Clz(Template):
#     def __init__(self):
#         super().__init__("1")
#         self.var_c = 1
#
#
# c = Clz()
# print(c.__dict__)
# c.add()
# print(c.__dict__)
# import re
# a = """
# total 17
# -rw-r--r-- 1 wsl wsl 1093 Aug 31 11:17 1.ps
# -rw-r--r-- 1 wsl wsl  552 Aug 31 11:17 2.ps
# -rw-r--r-- 1 wsl wsl  437 Aug 31 11:17 3.ps
# -rw-r--r-- 1 wsl wsl  199 Aug 27 11:58 a.c
# -rw-r--r-- 1 wsl wsl 1880 Aug 27 11:58 a.o
# -rw-r--r-- 1 wsl wsl 5360 Aug 31 11:39 temp.ps
# """
# p = re.compile(r"([a-z\-]+)\s+(\d+)\s+(.+)\s+(.+)\s+(.+)\s+(.+)\s+(.+)\s+(.+)\s+(.+)\s+", re.MULTILINE)
# r = p.findall(a)
# print(r)

def check_switch_case_condition(a, b):
    if callable(b):
        return b(a)
    return a == b